'use strict';

const fetchres = require('fetchres');
const logger = require('ft-next-logger').logger;
const signedFetch = require('signed-aws-es-fetch');
const elasticAgent = require('./utils/elastic-agent');

function formatDateRange(from, to) {
	const dateFrom = new Date();
	const dateTo = new Date();

	dateFrom.setDate(dateFrom.getDate() - from);
	dateTo.setDate(dateTo.getDate() - to);

	return {
		gte: dateFrom.toISOString(),
		lt: dateTo.toISOString()
	};
}

function formatPostBody(options) {
	return {
		size: 0,
		query: {
			bool: {
				must: [
					{
						term: { 'metadata.idV1': options.id }
					},
					{
						range: {
							'publishedDate': formatDateRange(options.range || 60, 1)
						}
					}
				]
			}
		},
		aggs: {
			tags: {
				nested: {
					path: 'metadata'
				},
				aggs: {
					filtered: {
						filter: {
							bool: {
								should: options.taxonomies.map(taxonomy => {
									return { term: { taxonomy: taxonomy } };
								})
							}
						},
						aggs: {
							suggestions: {
								terms: {
									field: 'idV1',
									exclude: options.id,
									size: options.size || 12,
									min_doc_count: 1
								},
								aggs: {
									prefLabel: {
										terms: {
											field: 'prefLabel',
											size: 1
										}
									},
									taxonomy: {
										terms: {
											field: 'taxonomy',
											size: 1
										}
									}
								}
							}
						}
					}
				}
			}
		}
	};
}

function mapResponseBucketsToTags(aggregations) {
	return aggregations.tags.filtered.suggestions.buckets.map(bucket => {
		return {
			idV1: bucket.key,
			taxonomy: bucket.taxonomy.buckets[0].key,
			prefLabel: bucket.prefLabel.buckets[0].key
		};
	});
}

module.exports = (options) => {
	const postBody = formatPostBody(options);

	return signedFetch('https://next-elastic.ft.com/v3_api_v2/item/_search', {
		retry: 0,
		body: JSON.stringify(postBody),
		method: 'POST',
		timeout: 3000,
		agent: elasticAgent
	})
		.then(response => {
			if (response.ok) {
				return fetchres.json(response);
			} else {
				logger.warn(`Failed getting related tags for: ${JSON.stringify(options, null, 2)}`);
				return {};
			}
		})
		.then(data => {
			// Log slow requests
			if (data.took > 200) {
				logger.warn(`Slow response for related tags query: ${data.took}ms`);
			}

			return data.aggregations
				? mapResponseBucketsToTags(data.aggregations)
				: [];
		});
};
